@@ -1,3 +1,21 @@
+/* Copyright 2007 Matthew Barnes
+ *
+ * This file is part of GNOME Video Arcade.
+ *
+ * GNOME Video Arcade is free software; you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
+ *
+ * GNOME Video Arcade is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #ifndef GVA_UI_H
 #define GVA_UI_H
 
@@ -10,6 +28,8 @@
 #define GVA_ACTION_AUTO_SAVE            (gva_ui_get_action ("auto-save"))
 #define GVA_ACTION_CONTENTS             (gva_ui_get_action ("contents"))
 #define GVA_ACTION_FULL_SCREEN          (gva_ui_get_action ("full-screen"))
+#define GVA_ACTION_GO_BACK              (gva_ui_get_action ("go-back"))
+#define GVA_ACTION_GO_FORWARD           (gva_ui_get_action ("go-forward"))
 #define GVA_ACTION_INSERT_FAVORITE      (gva_ui_get_action ("insert-favorite"))
 #define GVA_ACTION_PLAY_BACK            (gva_ui_get_action ("play-back"))
 #define GVA_ACTION_PROPERTIES           (gva_ui_get_action ("properties"))
@@ -23,6 +43,7 @@
 #define GVA_ACTION_VIEW_RESULTS         (gva_ui_get_action ("view-results"))
 
 /* Main Window */
+#define GVA_WIDGET_MAIN_PROGRESS_BAR         GW ("main-progress-bar")
 #define GVA_WIDGET_MAIN_PROPERTIES_BUTTON    GW ("main-properties-button")
 #define GVA_WIDGET_MAIN_START_GAME_BUTTON    GW ("main-start-game-button")
 #define GVA_WIDGET_MAIN_STATUSBAR            GW ("main-statusbar")
@@ -46,6 +67,15 @@
 #define GVA_WIDGET_PREFERENCES_FULL_SCREEN   GW ("preferences-full-screen")
 #define GVA_WIDGET_PREFERENCES_WINDOW        GW ("preferences-window")
 
+/* Properties Window */
+#define GVA_WIDGET_PROPERTIES_BACK_BUTTON    GW ("properties-back-button")
+#define GVA_WIDGET_PROPERTIES_CLOSE_BUTTON   GW ("properties-close-button")
+#define GVA_WIDGET_PROPERTIES_FORWARD_BUTTON GW ("properties-forward-button")
+#define GVA_WIDGET_PROPERTIES_HEADER         GW ("properties-header")
+#define GVA_WIDGET_PROPERTIES_HISTORY_TEXT_VIEW \
+                                            GW ("properties-history-text-view")
+#define GVA_WIDGET_PROPERTIES_WINDOW         GW ("properties-window")
+
 G_BEGIN_DECLS
 
 GtkAction *     gva_ui_get_action               (const gchar *action_name);
