@@ -1,7 +1,26 @@
+/* Copyright 2007 Matthew Barnes
+ *
+ * This file is part of GNOME Video Arcade.
+ *
+ * GNOME Video Arcade is free software; you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
+ *
+ * GNOME Video Arcade is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "gva-xmame.h"
 
 #include <errno.h>
 #include <stdarg.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <wait.h>
@@ -10,206 +29,95 @@
 #include <wordexp.h>
 #endif
 
+#include "gva-error.h"
+#include "gva-mame-process.h"
 #include "gva-preferences.h"
 
+typedef struct
+{
+        GvaXmameCallback callback;
+        gpointer user_data;
+        gint lineno;
+
+} XmameAsyncData;
+
+static XmameAsyncData *
+xmame_async_data_new (GvaXmameCallback callback,
+                      gpointer user_data)
+{
+        XmameAsyncData *data;
+
+        data = g_slice_new (XmameAsyncData);
+        data->callback = callback;
+        data->user_data = user_data;
+        data->lineno = 0;
+
+        return data;
+}
+
 static void
-xmame_post_game_analysis (GvaProcess *process)
+xmame_async_data_free (XmameAsyncData *data)
 {
-        gint exit_status;
-        GError *error = NULL;
-
-        exit_status = gva_xmame_wait_for_exit (process, &error);
-
-        if (error == NULL && exit_status != 0)
-                g_set_error (
-                        &error, GVA_ERROR, GVA_ERROR_XMAME,
-                        _("Child process exited with status (%d)"),
-                        exit_status);
-
-        if (error != NULL)
-        {
-                g_warning ("%s", error->message);
-                g_clear_error (&error);
-        }
-
-        gva_process_free (process);
-}
-
-static gboolean
-xmame_scan_for_error (const gchar *string, GError **error)
-{
-        gchar **lines;
-        guint n_lines, ii;
-
-        if (string == NULL)
-                return FALSE;
-
-        lines = g_strsplit_set (string, "\n", -1);
-        n_lines = g_strv_length (lines);
-
-        for (ii = 0; ii < n_lines; ii++)
-        {
-                if (strlen (g_strstrip (lines[ii])) == 0)
-                        continue;
-
-                if (g_ascii_strncasecmp (lines[ii], "error: ", 7) == 0)
-                {
-                        gchar *error_message;
-
-                        error_message = lines[ii] + 7;
-                        *error_message = g_ascii_toupper (*error_message);
-
-                        g_set_error (
-                                error, GVA_ERROR, GVA_ERROR_XMAME,
-                                "%s", error_message);
-
-                        break;
-                }
-        }
-
-        g_strfreev (lines);
-
-        return (ii < n_lines);
-}
-
-GQuark
-gva_error_quark (void)
-{
-        static GQuark quark = 0;
-
-        if (G_UNLIKELY (quark == 0))
-                quark = g_quark_from_static_string ("gva-error-quark");
-
-        return quark;
-}
-
-gchar *
-gva_xmame_get_executable (GError **error)
-{
-        gchar *executable;
-
-        executable = g_find_program_in_path ("xmame");
-
-        if (executable == NULL)
-                g_set_error (error, GVA_ERROR, GVA_ERROR_XMAME,
-                        _("Could not find xmame executable"));
-
-        return executable;
-}
-
-GvaProcess *
-gva_xmame_async_command (const gchar *arguments,
-                         GvaProcessNotify on_stdout,
-                         GvaProcessNotify on_stderr,
-                         GvaProcessNotify on_exit,
-                         gpointer user_data,
-                         GError **error)
-{
-        GvaProcess *process;
-        gchar *command_line;
-        gchar *executable;
-        gchar **argv;
-        gint standard_input;
-        gint standard_output;
-        gint standard_error;
-        GPid child_pid;
-        gboolean success;
-
-        g_return_val_if_fail (arguments != NULL, FALSE);
-
-        if ((executable = gva_xmame_get_executable (error)) == NULL)
-                return NULL;
-
-        command_line = g_strdup_printf ("%s %s", executable, arguments);
-        success = g_shell_parse_argv (command_line, NULL, &argv, error);
-        g_free (command_line);
-        g_free (executable);
-
-        if (!success)
-                return NULL;
-
-        success = gdk_spawn_on_screen_with_pipes (
-                gdk_screen_get_default (), NULL, argv, NULL,
-                G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &child_pid,
-                &standard_input, &standard_output, &standard_error,
-                error);
-
-        g_strfreev (argv);
-
-        if (!success)
-                return NULL;
-
-        return gva_process_new (
-                child_pid, standard_input, standard_output, standard_error,
-                on_stdout, on_stderr, on_exit, user_data, error);
+        g_slice_free (XmameAsyncData, data);
 }
 
 gint
 gva_xmame_command (const gchar *arguments,
-                   gchar **standard_output,
-                   gchar **standard_error,
+                   gchar ***stdout_lines,
+                   gchar ***stderr_lines,
                    GError **error)
 {
+        gchar **local_stdout_lines = NULL;
+        gchar **local_stderr_lines = NULL;
         GvaProcess *process;
-        gint exit_status;
+        gint status;
 
-        process = gva_xmame_async_command (
-                arguments, NULL, NULL, NULL, NULL, error);
+        process = gva_mame_process_spawn (arguments, error);
         if (process == NULL)
                 return -1;
 
-        exit_status = gva_xmame_wait_for_exit (process, error);
+        /* Wait for the process to exit. */
+        while (!gva_process_has_exited (process, &status))
+                g_main_context_iteration (NULL, TRUE);
 
-        if (exit_status >= 0 && standard_output != NULL)
-                *standard_output = gva_process_read_stdout (process);
+        local_stdout_lines = gva_process_stdout_read_lines (process);
+        local_stderr_lines = gva_process_stderr_read_lines (process);
 
-        if (exit_status >= 0 && standard_error != NULL)
-                *standard_error = gva_process_read_stderr (process);
-
-        gva_process_free (process);
-
-        return exit_status;
-}
-
-gint
-gva_xmame_wait_for_exit (GvaProcess *process, GError **error)
-{
-        gint exit_status;
-
-        g_return_val_if_fail (process != NULL, FALSE);
-
-        while (!gva_process_has_exited (process, &exit_status))
-                g_main_context_iteration (g_main_context_default (), TRUE);
-
-        if (xmame_scan_for_error (gva_process_peek_stderr (process), error))
-                return -1;
-
-        if (xmame_scan_for_error (gva_process_peek_stdout (process), error))
-                return -1;
-
-        if (WIFSIGNALED (exit_status))
+        if (process->error != NULL)
         {
-                g_set_error (
-                        error, GVA_ERROR, GVA_ERROR_XMAME,
-                        "Child process terminated: %s",
-                        g_strsignal (WTERMSIG (exit_status)));
-                return -1;
+                g_propagate_error (error, process->error);
+                status = -1;
+                goto fail;
         }
 
-        g_assert (WIFEXITED (exit_status));
-        return WEXITSTATUS (exit_status);
+        if (stdout_lines != NULL)
+                *stdout_lines = local_stdout_lines;
+        else
+                g_strfreev (local_stdout_lines);
+
+        if (stderr_lines != NULL)
+                *stderr_lines = local_stderr_lines;
+        else
+                g_strfreev (local_stderr_lines);
+
+        g_assert (WIFEXITED (status));
+        status = WEXITSTATUS (status);
+
+fail:
+        g_object_unref (process);
+
+        return status;
 }
 
 gchar *
 gva_xmame_get_version (GError **error)
 {
         gchar *version = NULL;
-        gchar *output;
         gchar **lines;
-        guint n_lines, ii;
+        guint num_lines, ii;
 
         /* Execute the command "${xmame} -version". */
-        if (gva_xmame_command ("-version", &output, NULL, error) != 0)
+        if (gva_xmame_command ("-version", &lines, NULL, error) != 0)
                 return NULL;
 
         /* Output is as follows:
@@ -217,17 +125,15 @@ gva_xmame_get_version (GError **error)
          * xmame (backend) version n.nnn (Mmm dd yyyy)
          */
 
-        lines = g_strsplit_set (output, "\n", -1);
-        n_lines = g_strv_length (lines);
-        g_free (output);
+        num_lines = g_strv_length (lines);
 
-        for (ii = 0; ii < n_lines && version == NULL; ii++)
+        for (ii = 0; ii < num_lines && version == NULL; ii++)
         {
                 if (strstr (lines[ii], "xmame") == NULL)
                         continue;
                 if (strstr (lines[ii], "version") == NULL)
                         continue;
-                version = g_strdup (lines[ii]);
+                version = g_strdup (g_strstrip (lines[ii]));
         }
 
         g_strfreev (lines);
@@ -235,13 +141,46 @@ gva_xmame_get_version (GError **error)
         return version;
 }
 
+guint
+gva_xmame_get_total_supported (GError **error)
+{
+        gchar **lines;
+        guint num_lines, ii;
+        guint total_supported = 0;
+
+        /* Execute the command "${xmame} -list". */
+        if (gva_xmame_command ("-list", &lines, NULL, error) != 0)
+                return 0;
+
+        /* Output is as follows:
+         *
+         * xmame currently supports:
+         * romname   romname   romname   romname   romname   ...
+         * romname   romname   romname   romname   romname   ...
+         * romname   romname   romname   romname   romname   ...
+         *
+         * Total Supported: nnnn
+         */
+
+        num_lines = g_strv_length (lines);
+
+        for (ii = 0; ii < num_lines; ii++)
+        {
+                if (g_str_has_prefix (lines[ii], "Total Supported:"))
+                        total_supported = strtoul (lines[ii] + 16, NULL, 10);
+        }
+
+        g_strfreev (lines);
+
+        return total_supported;
+}
+
 gchar *
 gva_xmame_get_config_value (const gchar *config_key, GError **error)
 {
         gchar *config_value = NULL;
-        gchar *output;
         gchar **lines;
-        guint n_lines, ii;
+        guint num_lines, ii;
 #ifdef HAVE_WORDEXP_H
         wordexp_t words;
 #endif
@@ -249,7 +188,7 @@ gva_xmame_get_config_value (const gchar *config_key, GError **error)
         g_return_val_if_fail (config_key != NULL, NULL);
 
         /* Execute the command "${xmame} -showconfig". */
-        if (gva_xmame_command ("-showconfig", &output, NULL, error) != 0)
+        if (gva_xmame_command ("-showconfig", &lines, NULL, error) != 0)
                 return NULL;
 
         /* Output is as follows:
@@ -260,11 +199,9 @@ gva_xmame_get_config_value (const gchar *config_key, GError **error)
          * ...
          */
 
-        lines = g_strsplit_set (output, "\n", -1);
-        n_lines = g_strv_length (lines);
-        g_free (output);
+        num_lines = g_strv_length (lines);
 
-        for (ii = 0; ii < n_lines; ii++)
+        for (ii = 0; ii < num_lines; ii++)
         {
                 if (g_str_has_prefix (lines[ii], config_key))
                 {
@@ -278,7 +215,7 @@ gva_xmame_get_config_value (const gchar *config_key, GError **error)
 
 #ifdef HAVE_WORDEXP_H
         if (config_value == NULL)
-                return NULL;
+                goto exit;
 
         /* xmame reports shell variables like $HOME in some of its
          * configuration values, so we need to expand them ourselves. */
@@ -302,6 +239,12 @@ gva_xmame_get_config_value (const gchar *config_key, GError **error)
         }
 #endif
 
+exit:
+        if (config_value == NULL)
+                g_set_error (
+                        error, GVA_ERROR, GVA_ERROR_XMAME,
+                        _("%s: No such configuration key"), config_key);
+
         return config_value;
 }
 
@@ -312,46 +255,17 @@ gva_xmame_has_config_value (const gchar *config_key)
         GError *error = NULL;
 
         config_value = gva_xmame_get_config_value (config_key, &error);
+        gva_error_handle (&error);
+
         if (config_value != NULL)
         {
                 g_free (config_value);
                 return TRUE;
         }
-        else if (error != NULL)
-        {
-                g_warning ("%s", error->message);
-                g_error_free (error);
-        }
 
         return FALSE;
 }
 
-GList *
-gva_xmame_get_available (GError **error)
-{
-        gchar *romname;
-        gchar *rompath;
-        GList *list = NULL;
-        GDir *dir;
-
-        rompath = gva_xmame_get_config_value ("rompath", error);
-        dir = (rompath != NULL) ? g_dir_open (rompath, 0, error) : NULL;
-        g_free (rompath);
-
-        if (dir == NULL)
-                return NULL;
-
-        while ((romname = g_strdup (g_dir_read_name (dir))) != NULL)
-        {
-                g_strdelimit (romname, ".", '\0');
-                list = g_list_prepend (list, romname);
-        }
-
-        g_dir_close (dir);
-
-        return g_list_reverse (list);
-}
-
 GHashTable *
 gva_xmame_get_input_files (GError **error)
 {
@@ -384,16 +298,16 @@ gva_xmame_get_input_files (GError **error)
 
                 if (channel != NULL)
                 {
-                        gchar romname[16];
+                        gchar name[16];
                         GIOStatus status;
 
                         status = g_io_channel_read_chars (
-                                channel, romname, sizeof (romname),
+                                channel, name, sizeof (name),
                                 NULL, &local_error);
                         if (status == G_IO_STATUS_NORMAL)
                                 g_hash_table_insert (
                                         hash_table, filename,
-                                        g_strdup (romname));
+                                        g_strdup (name));
                         g_io_channel_unref (channel);
                 }
 
@@ -415,70 +329,18 @@ exit:
         return hash_table;
 }
 
-GHashTable *
-gva_xmame_list_full (GError **error)
+GvaProcess *
+gva_xmame_list_xml (GError **error)
 {
-        GHashTable *hash_table = NULL;
-        gchar *output;
-        gchar **lines;
-        guint n_lines, ii;
-
-        /* Execute the command "${xmame} -listfull". */
-        if (gva_xmame_command ("-listfull", &output, NULL, error) != 0)
-                return NULL;
-
-        /* Output is as follows:
-         *
-         * name      description
-         * --------  -----------
-         * romname   "Full Game Title"
-         * romname   "Full Game Title"
-         * ...
-         *
-         *
-         * Total Supported: nnnn
-         */
-
-        hash_table = g_hash_table_new_full (
-                g_str_hash, g_str_equal,
-                (GDestroyNotify) g_free,
-                (GDestroyNotify) g_free);
-
-        lines = g_strsplit_set (output, "\n", -1);
-        n_lines = g_strv_length (lines);
-        g_assert (n_lines > 0);
-        g_free (output);
-
-        for (ii = 0; ii < n_lines; ii++)
-        {
-                gchar *key, *value, *cp;
-
-                cp = strchr (lines[ii], '\"');
-                if (cp == NULL || strchr (cp + 1, '\"') == NULL)
-                        continue;
-                value = g_strdup (g_strdelimit (cp, "\"", '\0') + 1);
-                key = g_strdup (g_strchomp (lines[ii]));
-
-                g_hash_table_insert (hash_table, key, value);
-        }
-
-        g_strfreev (lines);
-
-        return hash_table;
+        /* Execute the command "${xmame} -listxml". */
+        return gva_mame_process_spawn ("-listxml", error);
 }
 
-GHashTable *
-gva_xmame_verify_sample_sets (GError **error)
+static void
+xmame_verify_read (GvaProcess *process,
+                   XmameAsyncData *data)
 {
-        GHashTable *hash_table = NULL;
-        gchar *output;
-        gchar **lines;
-        guint n_lines, ii;
-
-        /* Execute the command "${xmame} -verifysamplesets". */
-        /* XXX What are the exit codes for this command? */
-        if (gva_xmame_command ("-verifysamplesets", &output, NULL, error) < 0)
-                return NULL;
+        gchar *line;
 
         /* Output is as follows:
          *
@@ -495,44 +357,103 @@ gva_xmame_verify_sample_sets (GError **error)
          * Not found: nnnn
          */
 
-        hash_table = g_hash_table_new_full (
-                g_str_hash, g_str_equal,
-                (GDestroyNotify) g_free,
-                (GDestroyNotify) g_free);
+        line = gva_process_stdout_read_line (process);
 
-        lines = g_strsplit_set (output, "\n", -1);
-        n_lines = g_strv_length (lines);
-        g_assert (n_lines > 4);
-        g_free (output);
+        /* Stop parsing when we see a blank line. */
+        if (data->lineno < 0 || *g_strchomp (line) == '\0')
+                data->lineno = -1;
+        else
+                data->lineno++;
 
-        for (ii = 2; ii < n_lines - 4; ii++)
+        if (data->lineno > 2)
         {
                 gchar **tokens;
 
-                tokens = g_strsplit_set (lines[ii], " ", 2);
+                tokens = g_strsplit_set (line, " ", 2);
                 if (g_strv_length (tokens) == 2)
                 {
-                        gchar *key, *value;
+                        gchar *name, *status;
 
-                        key = g_strdup (g_strstrip (tokens[0]));
-                        value = g_strdup (g_strstrip (tokens[1]));
-                        g_hash_table_insert (hash_table, key, value);
+                        name = g_strstrip (tokens[0]);
+                        status = g_strstrip (tokens[1]);
+
+                        data->callback (name, status, data->user_data);
                 }
                 g_strfreev (tokens);
         }
 
-        g_strfreev (lines);
-
-        return hash_table;
+        g_free (line);
 }
 
-gboolean
-gva_xmame_run_game (const gchar *romname, GError **error)
+static void
+xmame_verify_exit (GvaProcess *process,
+                   gint status,
+                   XmameAsyncData *data)
+{
+        xmame_async_data_free (data);
+}
+
+GvaProcess *
+gva_xmame_verify_romsets (GvaXmameCallback callback,
+                          gpointer user_data,
+                          GError **error)
+{
+        GvaProcess *process;
+        XmameAsyncData *data;
+
+        g_return_val_if_fail (callback != NULL, NULL);
+
+        /* Execute the command "${xmame} -verifyromsets". */
+        process = gva_mame_process_spawn ("-verifyromsets", error);
+        if (process == NULL)
+                return NULL;
+
+        data = xmame_async_data_new (callback, user_data);
+
+        g_signal_connect (
+                process, "stdout-ready",
+                G_CALLBACK (xmame_verify_read), data);
+        g_signal_connect (
+                process, "exited",
+                G_CALLBACK (xmame_verify_exit), data);
+
+        return process;
+}
+
+GvaProcess *
+gva_xmame_verify_samplesets (GvaXmameCallback callback,
+                             gpointer user_data,
+                             GError **error)
+{
+        GvaProcess *process;
+        XmameAsyncData *data;
+
+        g_return_val_if_fail (callback != NULL, NULL);
+
+        /* Execute the command "${xmame} -verifysamplesets". */
+        process = gva_mame_process_spawn ("-verifysamplesets", error);
+        if (process == NULL)
+                return NULL;
+
+        data = xmame_async_data_new (callback, user_data);
+
+        g_signal_connect (
+                process, "stdout-ready",
+                G_CALLBACK (xmame_verify_read), data);
+        g_signal_connect (
+                process, "exited",
+                G_CALLBACK (xmame_verify_exit), data);
+
+        return process;
+}
+
+GvaProcess *
+gva_xmame_run_game (const gchar *name, GError **error)
 {
         GvaProcess *process;
         GString *arguments;
 
-        g_return_val_if_fail (romname != NULL, FALSE);
+        g_return_val_if_fail (name != NULL, FALSE);
 
         arguments = g_string_sized_new (64);
 
@@ -552,29 +473,27 @@ gva_xmame_run_game (const gchar *romname, GError **error)
                         g_string_append (arguments, "-nofullscreen ");
         }
 
-        g_string_append_printf (arguments, "%s", romname);
+        g_string_append_printf (arguments, "%s", name);
 
-        /* Execute the command "${xmame} ${romname}". */
-        process = gva_xmame_async_command (
-                arguments->str, NULL, NULL, (GvaProcessNotify)
-                xmame_post_game_analysis, NULL, error);
+        /* Execute the command "${xmame} ${name}". */
+        process = gva_mame_process_spawn (arguments->str, error);
 
         g_string_free (arguments, TRUE);
 
-        return (process != NULL);
+        return process;
 }
 
-gboolean
-gva_xmame_record_game (const gchar *romname, const gchar *inpname,
+GvaProcess *
+gva_xmame_record_game (const gchar *name, const gchar *inpname,
                        GError **error)
 {
         GvaProcess *process;
         GString *arguments;
 
-        g_return_val_if_fail (romname != NULL, FALSE);
+        g_return_val_if_fail (name != NULL, FALSE);
 
         if (inpname == NULL)
-                inpname = romname;
+                inpname = name;
 
         arguments = g_string_sized_new (64);
 
@@ -589,20 +508,18 @@ gva_xmame_record_game (const gchar *romname, const gchar *inpname,
                         g_string_append (arguments, "-nofullscreen ");
         }
 
-        g_string_append_printf (arguments, "-record %s %s", inpname, romname);
+        g_string_append_printf (arguments, "-record %s %s", inpname, name);
 
-        /* Execute the command "${xmame} -record ${inpname} ${romname}". */
-        process = gva_xmame_async_command (
-                arguments->str, NULL, NULL, (GvaProcessNotify)
-                xmame_post_game_analysis, NULL, error);
+        /* Execute the command "${xmame} -record ${inpname} ${name}". */
+        process = gva_mame_process_spawn (arguments->str, error);
 
         g_string_free (arguments, TRUE);
 
-        return (process != NULL);
+        return process;
 }
 
-gboolean
-gva_xmame_playback_game (const gchar *romname, const gchar *inpname,
+GvaProcess *
+gva_xmame_playback_game (const gchar *name, const gchar *inpname,
                          GError **error)
 {
         GvaProcess *process;
@@ -626,22 +543,23 @@ gva_xmame_playback_game (const gchar *romname, const gchar *inpname,
         g_string_append_printf (arguments, "-playback %s", inpname);
 
         /* Execute the command "${xmame} -playback ${inpname}". */
-        process = gva_xmame_async_command (
-                arguments->str, NULL, NULL, (GvaProcessNotify)
-                xmame_post_game_analysis, NULL, error);
+        process = gva_mame_process_spawn (arguments->str, error);
 
         g_string_free (arguments, TRUE);
 
         if (process == NULL)
-                return FALSE;
+                return NULL;
 
         /* xmame asks the user to press return before it will start playing
          * back the game, so we have to supply the expected keystroke. */
-        return gva_process_write_stdin (process, "\n", 1, error);
+        if (!gva_process_write_stdin (process, "\n", 1, error))
+                return NULL;  /* FIXME Abort the process */
+
+        return process;
 }
 
 gboolean
-gva_xmame_clear_state (const gchar *romname, GError **error)
+gva_xmame_clear_state (const gchar *name, GError **error)
 {
         gchar *basename;
         gchar *directory;
@@ -652,7 +570,7 @@ gva_xmame_clear_state (const gchar *romname, GError **error)
         if (directory == NULL)
                 return FALSE;
 
-        basename = g_strdup_printf ("%s.sta", romname);
+        basename = g_strdup_printf ("%s.sta", name);
         filename = g_build_filename (directory, basename, NULL);
 
         if (g_file_test (filename, G_FILE_TEST_IS_REGULAR))
